@@ -43,7 +43,8 @@ namespace ir {
   ///////////////////////////////////////////////////////////////////////////
 
   Function::Function(const std::string &name, const Unit &unit, Profile profile) :
-    name(name), unit(unit), profile(profile), simdWidth(0), useSLM(false), slmSize(0), stackSize(0)
+    name(name), unit(unit), profile(profile), simdWidth(0), useSLM(false), slmSize(0), stackSize(0),
+    wgBroadcastSLM(-1), tidMapSLM(-1), useDeviceEnqueue(false)
   {
     initProfile(*this);
     samplerSet = GBE_NEW(SamplerSet);
@@ -61,8 +62,42 @@ namespace ir {
     return unit.getPointerFamily();
   }
 
-  void Function::addLoop(const vector<LabelIndex> &bbs, const vector<std::pair<LabelIndex, LabelIndex>> &exits) {
-    loops.push_back(GBE_NEW(Loop, bbs, exits));
+  uint32_t Function::getOclVersion(void) const {
+    return unit.getOclVersion();
+  }
+
+  void Function::addLoop(LabelIndex preheader,
+                        int parent,
+                        const vector<LabelIndex> &bbs,
+                        const vector<std::pair<LabelIndex, LabelIndex>> &exits) {
+    loops.push_back(GBE_NEW(Loop, preheader, parent, bbs, exits));
+  }
+
+  int Function::getLoopDepth(LabelIndex Block) const{
+    if (loops.size() == 0) return 0;
+
+    int LoopIndex = -1;
+    int LoopDepth = 0;
+    // get innermost loop
+    for (int Idx = loops.size()-1; Idx >= 0; Idx--) {
+      Loop *lp = loops[Idx];
+      vector<LabelIndex> &Blocks = lp->bbs;
+      bool Found = (std::find(Blocks.begin(), Blocks.end(), Block) != Blocks.end());
+      if (Found) {
+        LoopIndex = Idx;
+        break;
+      }
+    }
+
+    if (LoopIndex != -1) {
+      int LoopId = LoopIndex;
+      do {
+        LoopId = loops[LoopId]->parent;
+        LoopDepth++;
+      } while(LoopId != -1);
+    }
+
+    return LoopDepth;
   }
 
   void Function::checkEmptyLabels(void) {
@@ -119,6 +154,8 @@ namespace ir {
       for (auto &y : x->bbs)
         y = labelMap[y];
 
+      x->preheader = labelMap[x->preheader];
+
       for (auto &z : x->exits) {
         z.first = labelMap[z.first];
         z.second = labelMap[z.second];
@@ -320,6 +357,7 @@ namespace ir {
           out << "structure." << input.size;
         break;
         case FunctionArgument::IMAGE: out << "image"; break;
+        case FunctionArgument::PIPE: out << "pipe"; break;
         default: break;
       }
       out << " %" << input.reg << " " << input.name << std::endl;
@@ -392,7 +430,7 @@ namespace ir {
   LabelIndex BasicBlock::getLabelIndex(void) const {
     const Instruction *first = this->getFirstInstruction();
     const LabelInstruction *label = cast<LabelInstruction>(first);
-    return label->getLabelIndex();
+    return label?label->getLabelIndex():LabelIndex(-1);
   }
 
 } /* namespace ir */
