@@ -53,29 +53,36 @@ namespace ir {
     static const Register lsize0 = Register(9);    // get_local_size(0)
     static const Register lsize1 = Register(10);   // get_local_size(1)
     static const Register lsize2 = Register(11);   // get_local_size(2)
-    static const Register gsize0 = Register(12);   // get_global_size(0)
-    static const Register gsize1 = Register(13);   // get_global_size(1)
-    static const Register gsize2 = Register(14);   // get_global_size(2)
-    static const Register goffset0 = Register(15); // get_global_offset(0)
-    static const Register goffset1 = Register(16); // get_global_offset(1)
-    static const Register goffset2 = Register(17); // get_global_offset(2)
-    static const Register stackptr = Register(18); // stack pointer
-    static const Register stackbuffer = Register(19); // stack buffer base address.
-    static const Register blockip = Register(20);  // blockip
-    static const Register barrierid = Register(21);// barrierid
-    static const Register threadn = Register(22);  // number of threads
-    static const Register workdim = Register(23);  // work dimention.
-    static const Register zero = Register(24);     //  scalar register holds zero.
-    static const Register one = Register(25);     //  scalar register holds one. 
-    static const Register retVal = Register(26);   // helper register to do data flow analysis.
-    static const Register slmoffset = Register(27);  // Group's SLM offset in total 64K SLM
-    static const Register printfbptr = Register(28); // printf buffer address .
-    static const Register printfiptr = Register(29); // printf index buffer address.
+    static const Register enqlsize0 = Register(12);    // get_local_size(0)
+    static const Register enqlsize1 = Register(13);   // get_local_size(1)
+    static const Register enqlsize2 = Register(14);   // get_local_size(2)
+    static const Register gsize0 = Register(15);   // get_global_size(0)
+    static const Register gsize1 = Register(16);   // get_global_size(1)
+    static const Register gsize2 = Register(17);   // get_global_size(2)
+    static const Register goffset0 = Register(18); // get_global_offset(0)
+    static const Register goffset1 = Register(19); // get_global_offset(1)
+    static const Register goffset2 = Register(20); // get_global_offset(2)
+    static const Register stackptr = Register(21); // stack pointer
+    static const Register stackbuffer = Register(22); // stack buffer base address.
+    static const Register blockip = Register(23);  // blockip
+    static const Register barrierid = Register(24);// barrierid
+    static const Register threadn = Register(25);  // number of threads
+    static const Register workdim = Register(26);  // work dimention.
+    static const Register zero = Register(27);     //  scalar register holds zero.
+    static const Register one = Register(28);     //  scalar register holds one.
+    static const Register retVal = Register(29);   // helper register to do data flow analysis.
     static const Register dwblockip = Register(30);  // blockip
-    static const Register laneid = Register(31);  // lane id.
-    static const Register invalid = Register(32);  // used for valid comparation.
-    static const Register btiUtil = Register(33);  // used for mixed pointer as bti utility.
-    static const uint32_t regNum = 34;             // number of special registers
+    static const Register profilingbptr = Register(31); // buffer addr for profiling.
+    static const Register profilingts0 = Register(32); // timestamp for profiling.
+    static const Register profilingts1 = Register(33); // timestamp for profiling.
+    static const Register profilingts2 = Register(34); // timestamp for profiling.
+    static const Register profilingts3 = Register(35); // timestamp for profiling.
+    static const Register profilingts4 = Register(36); // timestamp for profiling.
+    static const Register threadid = Register(37); // the thread id of this thread.
+    static const Register constant_addrspace = Register(38);  // starting address of program-scope constant
+    static const Register stacksize = Register(39); // stack buffer total size
+    static const Register enqueuebufptr = Register(40); // enqueue buffer address .
+    static const uint32_t regNum = 41;             // number of special registers
     extern const char *specialRegMean[];           // special register name.
   } /* namespace ocl */
 
