@@ -216,19 +216,18 @@ OVERLOADABLE half native_sin(half x);
 OVERLOADABLE half native_sqrt(half x);
 OVERLOADABLE half native_tan(half x);
 
-
 // half accuracy
-#define half_cos cos
-#define half_divide native_divide
-#define half_exp native_exp
-#define half_exp2 native_exp2
-#define half_exp10 native_exp10
-#define half_log native_log
-#define half_log2 native_log2
-#define half_log10 native_log10
-#define half_powr powr
-#define half_recip native_recip
-#define half_rsqrt native_rsqrt
-#define half_sin sin
-#define half_sqrt native_sqrt
-#define half_tan tan
+OVERLOADABLE float half_cos(float x);
+OVERLOADABLE float half_divide(float x, float y);
+OVERLOADABLE float half_exp(float x);
+OVERLOADABLE float half_exp2(float x);
+OVERLOADABLE float half_exp10(float x);
+OVERLOADABLE float half_log(float x);
+OVERLOADABLE float half_log2(float x);
+OVERLOADABLE float half_log10(float x);
+OVERLOADABLE float half_powr(float x, float y);
+OVERLOADABLE float half_recip(float x);
+OVERLOADABLE float half_rsqrt(float x);
+OVERLOADABLE float half_sin(float x);
+OVERLOADABLE float half_sqrt(float x);
+OVERLOADABLE float half_tan(float x);
