@@ -334,6 +334,7 @@ cl_program_create_from_binary(cl_context             ctx,
   else if (isGenBinary((unsigned char*)program->binary)) {
     program->opaque = interp_program_new_from_binary(program->ctx->devices[0]->device_id, program->binary, program->binary_sz);
     if (UNLIKELY(program->opaque == NULL)) {
+      DEBUGP(DL_ERROR, "Incompatible binary, please delete the binary and generate again.");
       err = CL_INVALID_PROGRAM;
       goto error;
     }
@@ -370,6 +371,7 @@ cl_program_create_with_built_in_kernles(cl_context     ctx,
                                   cl_int *             errcode_ret)
 {
   cl_int err = CL_SUCCESS;
+  cl_program built_in_prgs = NULL;
 
   assert(ctx);
   INVALID_DEVICE_IF (num_devices != 1);
@@ -381,54 +383,24 @@ cl_program_create_with_built_in_kernles(cl_context     ctx,
   extern size_t cl_internal_built_in_kernel_str_size;
   char* p_built_in_kernel_str =cl_internal_built_in_kernel_str;
 
-  ctx->built_in_prgs = cl_program_create_from_binary(ctx, 1,
-                                                          &ctx->devices[0],
-                                                          (size_t*)&cl_internal_built_in_kernel_str_size,
-                                                          (const unsigned char **)&p_built_in_kernel_str,
-                                                          &binary_status, &err);
-  if (!ctx->built_in_prgs)
+  built_in_prgs = cl_program_create_from_binary(ctx, 1,
+                                                &ctx->devices[0],
+                                                (size_t*)&cl_internal_built_in_kernel_str_size,
+                                                (const unsigned char **)&p_built_in_kernel_str,
+                                                &binary_status, &err);
+  if (!built_in_prgs)
     return NULL;
 
-  err = cl_program_build(ctx->built_in_prgs, NULL);
+  err = cl_program_build(built_in_prgs, NULL);
   if (err != CL_SUCCESS)
     return NULL;
 
-  ctx->built_in_prgs->is_built = 1;
-
-  char delims[] = ";";
-  char* saveptr = NULL;
-  char* local_kernel_names;
-  char* kernel = NULL;
-  char* matched_kernel;
-  int i = 0;
-
-  //copy the content to local_kernel_names to protect the kernel_names.
-  TRY_ALLOC(local_kernel_names, cl_calloc(strlen(kernel_names)+1, sizeof(char) ) );
-  memcpy(local_kernel_names, kernel_names, strlen(kernel_names)+1);
-
-  kernel = strtok_r( local_kernel_names, delims , &saveptr);
-  while( kernel != NULL ) {
-    matched_kernel = strstr(ctx->devices[0]->built_in_kernels, kernel);
-    if(matched_kernel){
-      for (i = 0; i < ctx->built_in_prgs->ker_n; ++i) {
-        assert(ctx->built_in_prgs->ker[i]);
-        const char *ker_name = cl_kernel_get_name(ctx->built_in_prgs->ker[i]);
-        if (ker_name != NULL && strcmp(ker_name, kernel) == 0) {
-          break;
-        }
-      }
-
-      ctx->built_in_kernels[i] = cl_program_create_kernel(ctx->built_in_prgs, kernel, NULL);
-    }
-    kernel = strtok_r((char*)saveptr , delims, &saveptr );
-  }
-
-  cl_free(local_kernel_names);
+  built_in_prgs->is_built = 1;
 
 exit:
   if (errcode_ret)
     *errcode_ret = err;
-  return ctx->built_in_prgs;
+  return built_in_prgs;
 error:
   goto exit;
 
@@ -457,7 +429,7 @@ cl_program_create_from_llvm(cl_context ctx,
       goto error;
   }
 
-  program->opaque = compiler_program_new_from_llvm(ctx->devices[0]->device_id, file_name, NULL, NULL, NULL, program->build_log_max_sz, program->build_log, &program->build_log_sz, 1, NULL);
+  program->opaque = compiler_program_new_from_llvm_file(ctx->devices[0]->device_id, file_name, program->build_log_max_sz, program->build_log, &program->build_log_sz);
   if (UNLIKELY(program->opaque == NULL)) {
     err = CL_INVALID_PROGRAM;
     goto error;
@@ -674,7 +646,8 @@ cl_program_build(cl_program p, const char *options)
     memcpy(p->bin + copyed, interp_kernel_get_code(opaque), sz);
     copyed += sz;
   }
-  if ((err = get_program_global_data(p)) != CL_SUCCESS)
+  uint32_t ocl_version = interp_kernel_get_ocl_version(interp_program_get_kernel(p->opaque, 0));
+  if (ocl_version >= 200 && (err = get_program_global_data(p)) != CL_SUCCESS)
     goto error;
 
   p->is_built = 1;
@@ -783,7 +756,8 @@ cl_program_link(cl_context            context,
     copyed += sz;
   }
 
-  if ((err = get_program_global_data(p)) != CL_SUCCESS)
+  uint32_t ocl_version = interp_kernel_get_ocl_version(interp_program_get_kernel(p->opaque, 0));
+  if (ocl_version >= 200 && (err = get_program_global_data(p)) != CL_SUCCESS)
     goto error;
 
 done:
